@@ -7,10 +7,16 @@
 #' @param annotate_feature Logical variable that specifies, whether to add feature values on the tile. This especially make sense in case of numeric features. Default value is \code{FALSE}.
 #' @param abbreviation Logical variable that specifies, whether use abbreviations for languages specified in the package. Default value is \code{TRUE}.
 #' @param hide_languages Character variable that specifies, which languages should be removed from the template.
-#' @param rename_languages This variable maps old language names to their corresponding new names. It can be represented as either:
+#' @param rename_languages This variable maps old language names to their corresponding new names. It can be represented as:
 #' \itemize{
-#' \item{A named vector, where names are the old language names and values are the corresponding new language names.}
-#' \item{A data frame with two columns: \code{language} (the old language names) and \code{new_language_name} (the corresponding new language names).}}
+#' \item{either a named vector, where names are the old language names and values are the corresponding new language names.}
+#' \item{or a data frame with two columns: \code{language} (the old language names) and \code{new_language_name} (the corresponding new language names).}}
+#' @param tile_colors Character variable that specifies the tile color based on variable levels. Behavior differs depending on the type of the feature variable.
+#' \itemize{
+#' \item{For numeric variables, it can be filled with a vector of 2 or 3 colors defining the lower, upper, and middle segments of the palette.}
+#' \item{For categorical variables, it can be filled with a vector of colors with the length equal to the number of possible values in the feature variable.}
+#' \item{The variable can also use palettes from the \code{RColorBrewer} and \code{viridis} packages.}}
+#' @param palette_reverse Logical variable that specifies, whether the colors in palettes from the \code{RColorBrewer} and \code{viridis} packages should be used in reverse order.
 #'
 #' @returns a `ggplot2` object
 #' @export
@@ -39,6 +45,9 @@
 #' @importFrom ggplot2 element_text
 #' @importFrom ggplot2 guides
 #' @importFrom grDevices col2rgb
+#' @importFrom scales hue_pal
+#' @importFrom scales col_factor
+#' @importFrom scales col_numeric
 #' @export
 
 ec_tile_map <- function(data = NULL,
@@ -48,7 +57,9 @@ ec_tile_map <- function(data = NULL,
                         annotate_feature = FALSE,
                         abbreviation = TRUE,
                         hide_languages = NULL,
-                        rename_languages = NULL) {
+                        rename_languages = NULL,
+                        tile_colors = NULL,
+                        palette_reverse = FALSE) {
 
   # arguments check ---------------------------------------------------------
 
@@ -90,7 +101,7 @@ ec_tile_map <- function(data = NULL,
         names(rename_languages) %in% RCaucTile::ec_languages$language)
   }
 
-# restructure rename_languages --------------------------------------------
+  # restructure rename_languages --------------------------------------------
 
   if("character" %in% class(rename_languages)){
     rename_languages <- as.data.frame(rename_languages)
@@ -98,7 +109,7 @@ ec_tile_map <- function(data = NULL,
     rename_languages$language <- rownames(rename_languages)
   }
 
-# redefine title_position -------------------------------------------------
+  # redefine title_position -------------------------------------------------
 
   if(title_position == "left") {
     title_position <- 0
@@ -128,12 +139,11 @@ ec_tile_map <- function(data = NULL,
       "The argument 'feature_column' should be a character vector with one value" =
         length(feature_column) == 1)
 
-# merge EC dataset with data provided by a user ---------------------------
+    # merge EC dataset with data provided by a user ---------------------------
 
     for_plot <- merge(RCaucTile::ec_languages, data, all.x = TRUE)
 
-
-# rename languages --------------------------------------------------------
+    # rename languages --------------------------------------------------------
 
     if(!is.null(rename_languages)){
       for_plot <- merge(for_plot, rename_languages, all.x = TRUE)
@@ -141,24 +151,20 @@ ec_tile_map <- function(data = NULL,
                                   for_plot$language,
                                   for_plot$new_language_name)
       for_plot$abbreviation <- ifelse(is.na(for_plot$new_language_name),
-                                  for_plot$abbreviation,
-                                  for_plot$new_language_name)
+                                      for_plot$abbreviation,
+                                      for_plot$new_language_name)
     }
 
-# hide languages ----------------------------------------------------------
+    # hide languages ----------------------------------------------------------
 
     if(!is.null(hide_languages)){
       for_plot <- for_plot[!(for_plot$language %in% hide_languages), ]
     }
 
-# create a column with the name feature if there is no one ----------------
+    # create a column with the name feature if there is no one ----------------
 
     names(for_plot)[names(for_plot) == feature_column] <- "feature"
 
-    # add an 'alpha' column for the cases when there are NAs in data ----------
-
-    for_plot$alpha <- ifelse(is.na(for_plot$feature), 0.2, 1)
-
     # change labels to abbreviations ------------------------------------------
 
     if(isTRUE(abbreviation)){
@@ -182,13 +188,17 @@ ec_tile_map <- function(data = NULL,
                       title = title,
                       title_position = title_position,
                       annotate_feature = annotate_feature,
-                      abbreviation = abbreviation)
+                      abbreviation = abbreviation,
+                      tile_colors = tile_colors,
+                      palette_reverse = palette_reverse)
     } else {
       ec_tile_categorical(data = for_plot,
                           title = title,
                           title_position = title_position,
                           annotate_feature = annotate_feature,
-                          abbreviation = abbreviation)
+                          abbreviation = abbreviation,
+                          tile_colors = tile_colors,
+                          palette_reverse = palette_reverse)
     }
   }
 }
@@ -246,23 +256,85 @@ ec_tile_numeric <- function(data,
                             title,
                             title_position,
                             annotate_feature,
-                            abbreviation){
+                            abbreviation,
+                            tile_colors,
+                            palette_reverse){
+
   # fake variables for R CMD check to be succeedded -------------------------
 
   x <- NULL
   y <- NULL
   feature <- NULL
   alpha <- NULL
+  text_color <- NULL
+  language <- NULL
 
   # load data ---------------------------------------------------------------
 
   for_plot <- data
 
-  # add a 'text_color' column for the text colors ---------------------------
+  # add colors for tiles and text -------------------------------------------
+
+  transform <- ifelse(palette_reverse, "reverse", "identity")
+
+  possible_palettes <- c("viridis", "inferno", "magma", "plasma", "BrBG",
+                         "PiYG", "PRGn", "PuOr", "RdBu", "RdGy", "RdYlBu",
+                         "RdYlGn", "Spectral", "Accent", "Dark2", "Paired",
+                         "Pastel1", "Pastel2", "Set1", "Set2", "Set3", "Blues",
+                         "BuGn", "BuPu", "GnBu", "Greens", "Greys", "Oranges",
+                         "OrRd", "PuBu", "PuBuGn", "PuRd", "Purples", "RdPu",
+                         "Reds", "YlGn", "YlGnBu", "YlOrBr", "YlOrRd")
+
+  if(length(tile_colors) <= 1){
+
+    if(is.null(tile_colors)){
+      scales::col_numeric(palette = "Blues",
+                          na.color = "grey95",
+                          reverse = palette_reverse,
+                          domain = c(min(for_plot$feature,
+                                         na.rm = TRUE),
+                                     max(for_plot$feature,
+                                         na.rm = TRUE)))(for_plot$feature) ->
+        for_plot$tile_color
+    } else if(tile_colors %in% possible_palettes){
+      scales::col_numeric(palette = tile_colors,
+                          na.color = "grey95",
+                          reverse = palette_reverse,
+                          domain = c(min(for_plot$feature,
+                                         na.rm = TRUE),
+                                     max(for_plot$feature,
+                                         na.rm = TRUE)))(for_plot$feature) ->
+        for_plot$tile_color
+    } else {
+      stop("We expect that if the variable 'tile_colors' is of length of one it is the name of 'ColorBrewer' or 'viridis' palettes")
+    }
+  } else if(length(tile_colors) > 3){
+
+    stop("We expect that the variable 'tile_colors' is either  a vector of length of 2 or 3 or the name of 'ColorBrewer' or 'viridis' palettes")
+
+  } else if(length(tile_colors) %in% c(2, 3)){
+
+    for_color_check <- check_colors(tile_colors)
+    for_color_check[for_color_check %in% FALSE] |>
+      names() ->
+      wrong_color_names
+
+    if(length(wrong_color_names) > 0){
+      paste("We found some cases with the wrong color names:",
+            paste(wrong_color_names, collapse = ", ")) |>
+        stop()
+    }
+
+    suppressWarnings(scales::gradient_n_pal(colours = tile_colors,
+                                            values = for_plot$feature)(for_plot$feature) ->
+                       for_plot$tile_color)
+
+  }
 
   for_plot$text_color <- ifelse(is.na(for_plot$feature),
-                                "grey60",
-                                "grey90")
+                                "grey80",
+                                define_annotation_color(for_plot$tile_color))
+
 
   # create a map ------------------------------------------------------------
 
@@ -271,21 +343,14 @@ ec_tile_numeric <- function(data,
                                  fill = feature,
                                  alpha = alpha)) +
     ggplot2::geom_tile(linewidth = 0) +
-    ggplot2::annotate(geom = "text",
-                      x = for_plot[for_plot$text_color == "grey90",]$x,
-                      y = for_plot[for_plot$text_color == "grey90",]$y,
-                      label = for_plot[for_plot$text_color == "grey90",]$language,
-                      color = "grey90") +
-    ggplot2::annotate(geom = "text",
-                      x = for_plot[for_plot$text_color == "grey60",]$x,
-                      y = for_plot[for_plot$text_color == "grey60",]$y,
-                      label = for_plot[for_plot$text_color == "grey60",]$language,
-                      color = "grey60") +
+    ggplot2::geom_text(ggplot2::aes(color = text_color, label = language)) +
     ggplot2::theme_void() +
-    ggplot2::labs(fill = NULL, color = NULL, title = title) +
+    ggplot2::labs(title = title, fill = NULL) +
     ggplot2::theme(legend.position = "bottom",
                    plot.title = ggplot2::element_text(hjust = title_position)) +
-    ggplot2::guides(alpha="none")
+    ggplot2::scale_fill_continuous(palette = tile_colors, na.value = "grey95", trans = transform)+
+    ggplot2::scale_color_manual(values = for_plot$text_color |> unique() |> sort())+
+    ggplot2::guides(alpha="none", color = "none")
 }
 
 ec_tile_categorical <- function(data,
@@ -293,7 +358,9 @@ ec_tile_categorical <- function(data,
                                 title_position,
                                 fill_by,
                                 annotate_feature,
-                                abbreviation){
+                                abbreviation,
+                                tile_colors,
+                                palette_reverse){
 
   # fake variables for R CMD check to be succeedded -------------------------
 
@@ -301,40 +368,93 @@ ec_tile_categorical <- function(data,
   y <- NULL
   feature <- NULL
   alpha <- NULL
+  text_color <- NULL
+  language <- NULL
 
   # load data ---------------------------------------------------------------
 
   for_plot <- data
 
-  # add a 'text_color' column for the text colors ---------------------------
+  # factor feature column ---------------------------------------------------
+
+  if(!is.factor(for_plot$feature)){
+    for_plot$feature <- factor(for_plot$feature)
+  }
+
+  # add colors for tiles and text -------------------------------------------
+  transform <- ifelse(palette_reverse, "reverse", "identity")
+
+  possible_palettes <- c("viridis", "inferno", "magma", "plasma", "BrBG",
+                         "PiYG", "PRGn", "PuOr", "RdBu", "RdGy", "RdYlBu",
+                         "RdYlGn", "Spectral", "Accent", "Dark2", "Paired",
+                         "Pastel1", "Pastel2", "Set1", "Set2", "Set3", "Blues",
+                         "BuGn", "BuPu", "GnBu", "Greens", "Greys", "Oranges",
+                         "OrRd", "PuBu", "PuBuGn", "PuRd", "Purples", "RdPu",
+                         "Reds", "YlGn", "YlGnBu", "YlOrBr", "YlOrRd")
+
+  if(length(tile_colors) <= 1){
+
+    if(is.null(tile_colors)){
+      scales::col_factor(palette = scales::hue_pal()(length(levels(for_plot$feature))),
+                         na.color = "grey95",
+                         domain = levels(for_plot$feature))(for_plot$feature) ->
+        for_plot$tile_color
+      tile_colors <- scales::hue_pal()(length(levels(for_plot$feature)))
+
+    } else if(tile_colors %in% possible_palettes){
+      scales::col_factor(palette = tile_colors,
+                         na.color = "grey95",
+                         reverse = palette_reverse,
+                         domain = for_plot$feature)(for_plot$feature) ->
+        for_plot$tile_color
+    } else {
+      stop("We expect that if the variable 'tile_colors' is of length of one it is the name of 'ColorBrewer' or 'viridis' palettes")
+    }
+
+    legend_colors <- scales::col_factor(palette = tile_colors,
+                                        na.color = "grey95",
+                                        reverse = palette_reverse,
+                                        domain = for_plot$feature)(levels(for_plot$feature))
+
+  } else if(length(tile_colors) > 1){
+    for_color_check <- check_colors(tile_colors)
+    for_color_check[for_color_check %in% FALSE] |>
+      names() ->
+      wrong_color_names
+
+    if(length(wrong_color_names) > 0){
+      paste("We found some cases with the wrong color names:",
+            paste(wrong_color_names, collapse = ", ")) |>
+        stop()
+    }
+    stopifnot("Argument 'tile_colors' should be either a vector of length equal to the number of levels in the 'feature' column, or a vector with ColorBrewer or viridis palettes" =
+                length(levels(for_plot$feature)) == length(tile_colors))
+
+        for_plot <- merge(for_plot,
+                          data.frame(feature = levels(for_plot$feature),
+                                     tile_color = tile_colors),
+                          all.x = TRUE)
+        legend_colors <- tile_colors
+  }
 
   for_plot$text_color <- ifelse(is.na(for_plot$feature),
-                                "grey60",
-                                "#000000")
+                                "grey80",
+                                define_annotation_color(for_plot$tile_color))
 
   # create a map ------------------------------------------------------------
 
   for_plot |>
-    ggplot2::ggplot(ggplot2::aes(x, y,
-                                 alpha = alpha)) +
-    ggplot2::geom_tile(linewidth = 0, color = "grey90") +
-    ggplot2::geom_tile(aes(fill = feature), linewidth = 0) +
-    ggplot2::annotate(geom = "text",
-                      x = for_plot[for_plot$text_color == "#000000",]$x,
-                      y = for_plot[for_plot$text_color == "#000000",]$y,
-                      label = for_plot[for_plot$text_color == "#000000",]$language,
-                      color = "#000000") +
-    ggplot2::annotate(geom = "text",
-                      x = for_plot[for_plot$text_color == "grey60",]$x,
-                      y = for_plot[for_plot$text_color == "grey60",]$y,
-                      label = for_plot[for_plot$text_color == "grey60",]$language,
-                      color = "grey60") +
+    ggplot2::ggplot(ggplot2::aes(x, y)) +
+    ggplot2::geom_tile(fill = "grey95", linewidth = 0) +
+    ggplot2::geom_tile(ggplot2::aes(fill = feature), linewidth = 0) +
+    ggplot2::geom_text(ggplot2::aes(color = text_color, label = language)) +
     ggplot2::theme_void() +
     ggplot2::labs(fill = NULL, color = NULL, title = title) +
     ggplot2::theme(legend.position = "bottom",
                    plot.title = ggplot2::element_text(hjust = title_position)) +
-    ggplot2::guides(alpha="none") +
-    ggplot2::scale_fill_discrete(na.translate = FALSE)
+    ggplot2::scale_fill_manual(values = legend_colors, na.translate = FALSE)+
+    ggplot2::scale_color_manual(values = for_plot$text_color |> unique() |> sort())+
+    ggplot2::guides(alpha="none", color = "none")
 }
 
 define_annotation_color <- function(colors){
@@ -348,5 +468,12 @@ define_annotation_color <- function(colors){
             colors_for_text$green*0.587 +
             colors_for_text$blue*0.114) > 160,
          "#000000",
-         "grey90")
+         "white")
+}
+
+check_colors <- function(x) {
+  sapply(x, function(color) {
+    tryCatch(is.matrix(grDevices::col2rgb(color)),
+             error = function(e) FALSE)
+  })
 }
